tmepath = os.getenv("TME_PATH_1_3_0")
package.path = package.path .. ";"..tmepath.."/contrib/viewerlib/?.lua;"..tmepath.."/contrib/terrameaddons/?.lua"

local cf   = require ("cellspace_functions")
local life = require ("life_patterns")
local view = require ("viewer")
local util = require ("util")

DEAD  = 0
EMPTY = 1
ALIVE = 2
DIM   = 30
STEPS = 100
WAIT  = 20

model = {
    initTime = 1,
    period   = 1,
    timesteps = STEPS
}

model.cellspace = CellularSpace { xdim = DIM, ydim = DIM }

model.cellspace:createNeighborhood { 
    strategy = "moore",
    self     = false,
    wrap     = true 
}

forEachCell (model.cellspace, function (cell) cell.state = DEAD end)

life.insert_pattern (model.cellspace, life.pulsar(), 10, 10 )
model.cellspace:synchronize()

model.map = view.Map {
                subject    =   model.cellspace,
                attributes =   {"state"},
                colors     =   "Blues",
                values     =   {DEAD, EMPTY, ALIVE}
}

model.timer = Timer {
                Event {time = model.initTime, period = model.period, 
                    action = function (ev)
                        -- plays the life game
                        model:game (ev:getTime())   
                        -- notifies the model to the viewed
                        model.cellspace:notify (ev:getTime())
                        -- waits for some time
                        util.wait(WAIT)
                    end
                }
            }

model.game    = function (model, t)
                   forEachCell(model.cellspace, function(cell)
                        alive = cf.countPastNeighbors(cell, "state", ALIVE)
                        if alive < 2 then
                            cell.state = DEAD
                        elseif alive > 3 then
                            cell.state = DEAD
                        elseif alive == 3 and cell.past.state == DEAD then
                            cell.state = ALIVE
                        end
                    end)
                    model.cellspace:synchronize()
end

-- run the model
model.timer:execute(STEPS)
